@@ -5,7 +5,7 @@
  * libradius.h	Structures and prototypes
  *		for the radius library.
  *
- * Version:	$Id: libradius.h,v 1.76 2004/05/28 07:20:15 phampson Exp $
+ * Version:	$Id: libradius.h,v 1.76.2.2 2005/09/01 15:55:13 nbk Exp $
  *
  */
 
@@ -291,7 +291,7 @@ char *		ip_ntoa(char *, uint32_t);
 uint32_t	ip_addr(const char *);
 char		*ifid_ntoa(char *buffer, size_t size, uint8_t *ifid);
 uint8_t		*ifid_aton(const char *ifid_str, uint8_t *ifid);
-char		*ipv6_ntoa(char *buffer, size_t size, void *ip6addr);
+const char	*ipv6_ntoa(char *buffer, size_t size, void *ip6addr);
 int		ipv6_addr(const char *ip6_str, void *ip6addr);
 char		*strNcpy(char *dest, const char *src, int n);
 void		rad_lowercase(char *str);
@@ -306,11 +306,14 @@ int		ascend_parse_filter(VALUE_PAIR *pair);
 void		print_abinary(VALUE_PAIR *vp, u_char *buffer, int len);
 #endif /*ASCEND_BINARY*/
 
-#ifdef HAVE_LOCAL_SNPRINTF
+/* snprintf.c */
+#ifndef HAVE_VSNPRINTF
 #include <stdarg.h>
-int snprintf(char *str, size_t count, const char *fmt, ...);
 int vsnprintf(char *str, size_t count, const char *fmt, va_list arg);
 #endif
+#ifndef HAVE_SNPRINTF
+int snprintf(char *str, size_t count, const char *fmt, ...);
+#endif
 
 /* random numbers in isaac.c */
 /* context of random number generator */
