@@ -1,7 +1,7 @@
 /*
  * misc.c	Various miscellaneous functions.
  *
- * Version:	$Id: misc.c,v 1.41.2.2 2004/10/04 15:26:46 aland Exp $
+ * Version:	$Id: misc.c,v 1.41.2.3 2005/08/24 14:37:51 nbk Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,7 @@
  */
 
 static const char rcsid[] =
-"$Id: misc.c,v 1.41.2.2 2004/10/04 15:26:46 aland Exp $";
+"$Id: misc.c,v 1.41.2.3 2005/08/24 14:37:51 nbk Exp $";
 
 #include	"autoconf.h"
 
@@ -349,7 +349,7 @@ uint8_t *ifid_aton(const char *ifid_str, uint8_t *ifid)
 /*
  *	Return an IPv6 address in standard colon notation
  */
-char *ipv6_ntoa(char *buffer, size_t size, void *ip6addr)
+const char *ipv6_ntoa(char *buffer, size_t size, void *ip6addr)
 {
 #if defined(HAVE_INET_NTOP) && defined(AF_INET6)
 	return inet_ntop(AF_INET6, (struct in6_addr *) ip6addr, buffer, size);
