@@ -1,7 +1,7 @@
 /*
  * radius.c	Functions to send/receive radius packets.
  *
- * Version:	$Id: radius.c,v 1.125.2.3 2004/12/19 20:11:54 aland Exp $
+ * Version:	$Id: radius.c,v 1.125.2.5 2005/08/19 19:43:46 aland Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@
  * Copyright 2000-2003  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: radius.c,v 1.125.2.3 2004/12/19 20:11:54 aland Exp $";
+static const char rcsid[] = "$Id: radius.c,v 1.125.2.5 2005/08/19 19:43:46 aland Exp $";
 
 #include	"autoconf.h"
 #include	"md5.h"
@@ -571,6 +571,11 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original,
 			  default:
 				  break;
 
+			  case PW_ACCOUNTING_REQUEST:
+			  case PW_ACCOUNTING_RESPONSE:
+				  memset(hdr->vector, 0, AUTH_VECTOR_LEN);
+				  break;
+
 			  case PW_AUTHENTICATION_ACK:
 			  case PW_AUTHENTICATION_REJECT:
 			  case PW_ACCESS_CHALLENGE:
@@ -1177,6 +1182,11 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original,
 			default:
 			  break;
 
+			case PW_ACCOUNTING_REQUEST:
+			case PW_ACCOUNTING_RESPONSE:
+			  memset(packet->data + 4, 0, AUTH_VECTOR_LEN);
+			  break;
+
 			case PW_AUTHENTICATION_ACK:
 			case PW_AUTHENTICATION_REJECT:
 			case PW_ACCESS_CHALLENGE:
@@ -1322,7 +1332,7 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original,
 			subptr = ptr + 4;
 			sublen = attrlen - 4;
 
-			while (sublen > 0) {
+			while (sublen >= 2) {
 				if (subptr[1] < 2) { /* too short */
 					break;
 				}
