@@ -1,7 +1,7 @@
 /*
  * acct.c	Accounting routines.
  *
- * Version:	$Id: acct.c,v 1.30 2004/04/14 19:53:58 aland Exp $
+ * Version:	$Id: acct.c,v 1.30.2.2 2005/07/11 13:47:45 nbk Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2000  Alan Curry <pacman@world.std.com>
  */
 
-static const char rcsid[] = "$Id: acct.c,v 1.30 2004/04/14 19:53:58 aland Exp $";
+static const char rcsid[] = "$Id: acct.c,v 1.30.2.2 2005/07/11 13:47:45 nbk Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -125,12 +125,27 @@ int rad_accounting(REQUEST *request)
 
 		/*
 		 *	Maybe one of the preacct modules has decided
-		 *	that a proxy should be used. If so, get out of
-		 *	here and send the proxied packet, but ONLY if
-		 *	there isn't one already...
+		 *	that a proxy should be used.
 		 */
-		if (pairfind(request->config_items, PW_PROXY_TO_REALM)) {
-			return reply;
+		if ((vp = pairfind(request->config_items, PW_PROXY_TO_REALM))) {
+			REALM *realm;
+
+			/*
+			 *	Check whether Proxy-To-Realm is
+			 *	a LOCAL realm.
+			 */
+			realm = realm_find(vp->strvalue, TRUE);
+			if (realm != NULL &&
+			    realm->acct_ipaddr == htonl(INADDR_NONE)) {
+				DEBUG("rad_accounting: Cancelling proxy to realm %s, as it is a LOCAL realm.", realm->realm);
+				pairdelete(&request->config_items, PW_PROXY_TO_REALM);
+			} else {
+				/*
+				 *	Don't reply to the NAS now because
+				 *	we have to send the proxied packet.
+				 */
+				return reply;
+			}
 		}
 	}
 
@@ -153,4 +168,3 @@ int rad_accounting(REQUEST *request)
 
 	return reply;
 }
-
