@@ -1,7 +1,7 @@
 /*
  * auth.c	User authentication.
  *
- * Version:	$Id: auth.c,v 1.136.2.1 2004/12/29 23:08:42 phampson Exp $
+ * Version:	$Id: auth.c,v 1.136.2.4 2005/08/25 00:10:28 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Jeff Carneal <jeff@apex.net>
  */
 
-static const char rcsid[] = "$Id: auth.c,v 1.136.2.1 2004/12/29 23:08:42 phampson Exp $";
+static const char rcsid[] = "$Id: auth.c,v 1.136.2.4 2005/08/25 00:10:28 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -65,38 +65,45 @@ char *auth_name(char *buf, size_t buflen, REQUEST *request, int do_cli) {
  */
 static int check_expiration(REQUEST *request)
 {
-	int result;
-	VALUE_PAIR *check_item = request->config_items;
+	VALUE_PAIR *check_item;
+	VALUE_PAIR *vp;
 
-	result = 0;
-	while (result == 0 && check_item != NULL) {
+	check_item = pairfind(request->config_items, PW_EXPIRATION);
 
-		/*
-		 *	Check expiration date if we are doing password aging.
-		 */
-		if (check_item->attribute == PW_EXPIRATION) {
-			/*
-			 *	Has this user's password expired?
-			 *
-			 *	If so, remove ALL reply attributes,
-			 *	and add our own Reply-Message, saying
-			 *	why they're being rejected.
-			 */
-			if (check_item->lvalue < (unsigned) time(NULL)) {
-				VALUE_PAIR *vp;
+	if (!check_item)  return 0;
 
-				result = -1;
-				vp = pairmake("Reply-Message",
-						"Password Has Expired\r\n",
-						T_OP_ADD);
-				pairfree(&request->reply->vps);
-				request->reply->vps = vp;
-				break;
-			}
-		}
-		check_item = check_item->next;
+	/*
+	 *	Has this user's password expired?
+	 *
+	 *	If so, remove ALL reply attributes,
+	 *	and add our own Reply-Message, saying
+	 *	why they're being rejected.
+	 */
+	if (((time_t) check_item->lvalue) <= request->timestamp) {
+		vp = pairmake("Reply-Message",
+			      "Password Has Expired\r\n",
+			      T_OP_ADD);
+		pairfree(&request->reply->vps);
+		request->reply->vps = vp;
+		return -1;
 	}
-	return result;
+
+#define EXP_TIMEOUT ((uint32_t) (((time_t) check_item->lvalue) - request->timestamp))
+	/*
+	 *	Otherwise, set the Session-Timeout based on expiration.
+	 */
+	vp = pairfind(request->reply->vps, PW_SESSION_TIMEOUT);
+	if (!vp) {
+		vp = pairmake("Session-Timeout", "0", T_OP_SET);
+		if (!vp) return -1; /* out of memory */
+		
+		vp->lvalue = EXP_TIMEOUT;
+		pairadd(&request->reply->vps, vp);
+	} else if (vp->lvalue > EXP_TIMEOUT) {
+		vp->lvalue = EXP_TIMEOUT;
+	} /* else Session-Timeout is smaller than Expiration, leave it alone */
+
+	return 0;
 }
 
 
@@ -256,6 +263,8 @@ int rad_check_password(REQUEST *request)
 	}
 
 	switch(auth_type) {
+		DICT_VALUE *dval;
+
 		case PW_AUTHTYPE_CRYPT:
 			/*
 			 *	Find the password sent by the user. It
@@ -342,8 +351,13 @@ int rad_check_password(REQUEST *request)
 			DEBUG2("auth: user supplied CHAP-Password matches local User-Password");
 			break;
 		default:
-			DEBUG2("auth: type \"%s\"",
-					dict_valbyattr(PW_AUTH_TYPE, auth_type)->name);
+			dval = dict_valbyattr(PW_AUTH_TYPE, auth_type);
+			if (dval) {
+				DEBUG2("auth: type \"%s\"", dval->name);
+			} else {
+				DEBUG2("auth: type UNKNOWN-%d", auth_type);
+			}
+
 			/*
 			 *	See if there is a module that handles
 			 *	this type, and turn the RLM_ return
@@ -884,7 +898,7 @@ autz_redo:
 	if (exec_program && exec_wait) {
 		r = radius_exec_program(exec_program, request,
 					exec_wait,
-					umsg, sizeof(umsg),
+					NULL, 0,
 					request->packet->vps, &tmp);
 		free(exec_program);
 		exec_program = NULL;
@@ -895,25 +909,31 @@ autz_redo:
 		pairmove(&request->reply->vps, &tmp);
 		pairfree(&tmp);
 
-		if (r != 0) {
+		if (r < 0) {
 			/*
 			 *	Error. radius_exec_program() returns -1 on
-			 *	fork/exec errors, or >0 if the exec'ed program
-			 *	had a non-zero exit status.
+			 *	fork/exec errors.
 			 */
-			if (umsg[0] == '\0') {
-				user_msg = "\r\nAccess denied (external check failed).";
-			} else {
-				user_msg = &umsg[0];
-			}
+			user_msg = "Access denied (external check failed)";
+			tmp = pairmake("Reply-Message", user_msg, T_OP_SET);
+			pairadd(&request->reply->vps, tmp);
 
 			request->reply->code = PW_AUTHENTICATION_REJECT;
-			tmp = pairmake("Reply-Message", user_msg, T_OP_SET);
-
-			pairadd(&request->reply->vps, tmp);
 			rad_authlog("Login incorrect (external check failed)",
-					request, 0);
+				    request, 0);
+			rad_postauth_reject(request);
 
+			return RLM_MODULE_REJECT;
+		}
+		if (r > 0) {
+			/*
+			 *	Reject. radius_exec_program() returns >0
+			 *	if the exec'ed program had a non-zero
+			 *	exit status.
+			 */
+			request->reply->code = PW_AUTHENTICATION_REJECT;
+			rad_authlog("Login incorrect (external check said so)",
+				    request, 0);
 			rad_postauth_reject(request);
 
 			return RLM_MODULE_REJECT;
