@@ -1,7 +1,7 @@
 /*
  * proxy.c	Proxy stuff.
  *
- * Version:	$Id: proxy.c,v 1.77 2004/05/28 21:45:07 aland Exp $
+ * Version:	$Id: proxy.c,v 1.77.2.2 2005/08/19 21:51:25 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Chris Parker <cparker@starnetusa.com>
  */
 
-static const char rcsid[] = "$Id: proxy.c,v 1.77 2004/05/28 21:45:07 aland Exp $";
+static const char rcsid[] = "$Id: proxy.c,v 1.77.2.2 2005/08/19 21:51:25 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -118,9 +118,11 @@ static void proxy_addinfo(REQUEST *request)
 static REALM *proxy_realm_ldb(REQUEST *request, const char *realm_name,
 			      int accounting)
 {
+	int		redone = 0;
 	REALM		*cl, *lb;
 	uint32_t	count;
 
+ redo:
 	lb = NULL;
 	count = 0;
 	for (cl = mainconfig.realms; cl; cl = cl->next) {
@@ -191,6 +193,25 @@ static REALM *proxy_realm_ldb(REQUEST *request, const char *realm_name,
 		}
 	} /* loop over the realms */
 
+	/*
+	 *	All are dead, see if we have to wake
+	 */
+	if (!redone && !lb && mainconfig.wake_all_if_all_dead) {
+		for (cl = mainconfig.realms; cl; cl = cl->next) {
+			if(strcasecmp(cl->realm,realm_name) == 0) {
+				if (!accounting && !cl->active) {
+					cl->active = TRUE;
+				}
+				else if (accounting &&
+					 !cl->acct_active) {
+					cl->acct_active = TRUE;
+				}
+			}
+		}
+		redone = 1;
+		goto redo;
+	}
+
 	/*
 	 *	Return the load-balanced realm.
 	 */
@@ -351,7 +372,8 @@ int proxy_send(REQUEST *request)
 	 *	Doing it here catches the case of proxied tunneled
 	 *	requests.
 	 */
-	if ((strippedname = pairfind(request->proxy->vps, PW_STRIPPED_USER_NAME)) != NULL) {
+	if (realm->striprealm == TRUE &&
+	   (strippedname = pairfind(request->proxy->vps, PW_STRIPPED_USER_NAME)) != NULL) {
 		/*
 		 *	If there's a Stripped-User-Name attribute in
 		 *	the request, then use THAT as the User-Name
