@@ -3,7 +3,7 @@
  *		Can also be installed as fingerd on the UNIX
  *		machine RADIUS runs on.
  *
- * Version:	$Id: radwho.c,v 1.44.2.1 2005/04/11 23:29:50 aland Exp $
+ * Version:	$Id: radwho.c,v 1.44.2.3 2005/08/18 17:26:42 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 
 static const char rcsid[] =
-"$Id: radwho.c,v 1.44.2.1 2005/04/11 23:29:50 aland Exp $";
+"$Id: radwho.c,v 1.44.2.3 2005/08/18 17:26:42 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -56,7 +56,7 @@ static const char rcsid[] =
 static const char *hdr1 =
 "Login      Name              What  TTY  When      From      Location";
 static const char *rfmt1 = "%-10.10s %-17.17s %-5.5s %s%-3d %-9.9s %-9.9s %-.19s%s";
-static const char *rfmt1r = "%s,%s,%s,%s%d,%s,%s,%s%s";
+static const char *rfmt1r = "%s,%s,%s,%s%u,%s,%s,%s%s";
 
 static const char *hdr2 =
 "Login      Port    What      When          From       Location";
@@ -296,7 +296,8 @@ int main(int argc, char **argv)
 	int radiusoutput = 0;	/* Radius attributes */
 	char *p, *q;
 	const char *portind;
-	int c, portno;
+	int c;
+	unsigned int portno;
 	char buffer[2048];
 	const char *user = NULL;
 	int user_cmp = 0;
@@ -533,7 +534,7 @@ int main(int argc, char **argv)
 			printf("NAS-IP-Address = %s\n",
 			       ip_hostname(buffer, sizeof(buffer),
 					   rt.nas_address));
-			printf("NAS-Port = %d\n", rt.nas_port);
+			printf("NAS-Port = %u\n", rt.nas_port);
 
 			switch (rt.proto) {
 				case 'S':
