@@ -1,7 +1,7 @@
 /*
  * threads.c	request threading support
  *
- * Version:	$Id: threads.c,v 1.77.2.2 2005/04/11 23:45:22 aland Exp $
+ * Version:	$Id: threads.c,v 1.77.2.6 2005/08/16 21:29:55 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -27,7 +27,26 @@
 
 #include <stdlib.h>
 #include <string.h>
+
+/*
+ *	Other OS's have sem_init, OS X doesn't.
+ */
+#ifndef DARWIN
 #include <semaphore.h>
+#else
+#include <mach/task.h>
+#include <mach/semaphore.h>
+
+#undef sem_t
+#define sem_t semaphore_t
+#undef sem_init
+#define sem_init(s,p,c) semaphore_create(mach_task_self(),s,SYNC_POLICY_FIFO,c)
+#undef sem_wait
+#define sem_wait(s) semaphore_wait(*s)
+#undef sem_post
+#define sem_post(s) semaphore_signal(*s)
+#endif
+
 #include <signal.h>
 
 #ifdef HAVE_SYS_WAIT_H
@@ -39,7 +58,7 @@
 #include "conffile.h"
 
 static const char rcsid[] =
-"$Id: threads.c,v 1.77.2.2 2005/04/11 23:45:22 aland Exp $";
+"$Id: threads.c,v 1.77.2.6 2005/08/16 21:29:55 aland Exp $";
 
 #define SEMAPHORE_LOCKED	(0)
 #define SEMAPHORE_UNLOCKED	(1)
@@ -226,6 +245,7 @@ static void request_enqueue(REQUEST *request, RAD_REQUEST_FUNP fun)
 	num_entries = ((thread_pool.queue_tail + thread_pool.queue_size) -
 		       thread_pool.queue_head) % thread_pool.queue_size;
 	if (num_entries == (thread_pool.queue_size - 1)) {
+		int i;
 		request_queue_t *new_queue;
 
 		/*
@@ -251,13 +271,19 @@ static void request_enqueue(REQUEST *request, RAD_REQUEST_FUNP fun)
 		 *	new one.
 		 */
 		new_queue = rad_malloc(sizeof(*new_queue) * thread_pool.queue_size * 2);
-		memcpy(new_queue, thread_pool.queue,
-		       sizeof(*new_queue) * thread_pool.queue_size);
-		memset(new_queue + sizeof(*new_queue) * thread_pool.queue_size,
+		/*
+		 *	Copy the queue element by element
+		 */
+		for (i = 0; i < thread_pool.queue_size; i++) {
+			new_queue[i] = thread_pool.queue[(i + thread_pool.queue_head) % thread_pool.queue_size];
+		}
+		memset(new_queue + thread_pool.queue_size,
 		       0, sizeof(*new_queue) * thread_pool.queue_size);
 
 		free(thread_pool.queue);
 		thread_pool.queue = new_queue;
+		thread_pool.queue_tail = ((thread_pool.queue_tail + thread_pool.queue_size) - thread_pool.queue_head) % thread_pool.queue_size;
+		thread_pool.queue_head = 0;
 		thread_pool.queue_size *= 2;
 	}
 
@@ -962,6 +988,11 @@ pid_t rad_fork(int exec_wait)
 
 	if (exec_wait) return fork();
 
+	/*
+	 *	Ensure that children are reaped always.
+	 */
+	reap_children();
+
 	/*
 	 *	Lock the mutex.
 	 */
