@@ -2,7 +2,7 @@
  * xlat.c	Translate strings.  This is the first version of xlat
  * 		incorporated to RADIUS
  *
- * Version:	$Id: xlat.c,v 1.72.2.3 2004/09/09 14:31:07 aland Exp $
+ * Version:	$Id: xlat.c,v 1.72.2.7 2005/08/29 16:19:41 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 static const char rcsid[] =
-"$Id: xlat.c,v 1.72.2.3 2004/09/09 14:31:07 aland Exp $";
+"$Id: xlat.c,v 1.72.2.7 2005/08/29 16:19:41 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -380,6 +380,7 @@ static void decode_attribute(const char **from, char **to, int freespace,
 	int stop=0, found=0, retlen=0;
 	int openbraces = *open;
 	xlat_t *c;
+	size_t namelen = sizeof(attrname);
 
 	p = *from;
 	q = *to;
@@ -397,7 +398,7 @@ static void decode_attribute(const char **from, char **to, int freespace,
 	/*
 	 *  Copy over the rest of the string.
 	 */
-	while ((*p) && (!stop)) {
+	while ((*p) && (!stop) && (namelen > 1)) {
 		switch(*p) {
 			/*
 			 *  Allow braces inside things, too.
@@ -440,6 +441,7 @@ static void decode_attribute(const char **from, char **to, int freespace,
 				*pa++ = *p++;
 				break;
 		}
+		namelen--;
 	}
 	*pa = '\0';
 
@@ -529,30 +531,23 @@ static void decode_attribute(const char **from, char **to, int freespace,
  */
 static int xlat_copy(char *out, int outlen, const char *in)
 {
-	int len = 0;
+	int freespace = outlen;
 
-	while (*in) {
-		/*
-		 *  Truncate, if too much.
-		 */
-		if (len >= outlen) {
-			break;
-		}
+	rad_assert(outlen > 0);
 
+	while ((*in) && (freespace > 1)) {
 		/*
 		 *  Copy data.
 		 *
 		 *  FIXME: Do escaping of bad stuff!
 		 */
-		*out = *in;
+		*(out++) = *(in++);
 
-		out++;
-		in++;
-		len++;
+		freespace--;
 	}
-
 	*out = '\0';
-	return len;
+
+	return (outlen - freespace); /* count does not include NUL */
 }
 
 /*
@@ -563,9 +558,10 @@ static int xlat_copy(char *out, int outlen, const char *in)
 int radius_xlat(char *out, int outlen, const char *fmt,
 		REQUEST *request, RADIUS_ESCAPE_STRING func)
 {
-	int i, c,freespace;
+	int c, len, freespace;
 	const char *p;
 	char *q;
+	char *nl;
 	VALUE_PAIR *tmp;
 	struct tm *TM, s_TM;
 	char tmpdt[40]; /* For temporary storing of dates */
@@ -668,9 +664,11 @@ int radius_xlat(char *out, int outlen, const char *fmt,
 				break;
 			case 'd': /* request day */
 				TM = localtime_r(&request->timestamp, &s_TM);
-				strftime(tmpdt,sizeof(tmpdt),"%d",TM);
-				strNcpy(q,tmpdt,freespace);
-				q += strlen(q);
+				len = strftime(tmpdt, sizeof(tmpdt), "%d", TM);
+				if (len > 0) {
+					strNcpy(q, tmpdt, freespace);
+					q += strlen(q);
+				}
 				p++;
 				break;
 			case 'f': /* Framed IP address */
@@ -690,9 +688,11 @@ int radius_xlat(char *out, int outlen, const char *fmt,
 				break;
 			case 'm': /* request month */
 				TM = localtime_r(&request->timestamp, &s_TM);
-				strftime(tmpdt,sizeof(tmpdt),"%m",TM);
-				strNcpy(q,tmpdt,freespace);
-				q += strlen(q);
+				len = strftime(tmpdt, sizeof(tmpdt), "%m", TM);
+				if (len > 0) {
+					strNcpy(q, tmpdt, freespace);
+					q += strlen(q);
+				}
 				p++;
 				break;
 			case 'n': /* NAS IP address */
@@ -708,7 +708,10 @@ int radius_xlat(char *out, int outlen, const char *fmt,
 				p++;
 				break;
 			case 't': /* request timestamp */
-				CTIME_R(&request->timestamp, q, freespace);
+				CTIME_R(&request->timestamp, tmpdt, sizeof(tmpdt));
+				nl = strchr(tmpdt, '\n');
+				if (nl) *nl = '\0';
+				strNcpy(q, tmpdt, freespace);
 				q += strlen(q);
 				p++;
 				break;
@@ -728,16 +731,20 @@ int radius_xlat(char *out, int outlen, const char *fmt,
 				break;
 			case 'D': /* request date */
 				TM = localtime_r(&request->timestamp, &s_TM);
-				strftime(tmpdt,sizeof(tmpdt),"%Y%m%d",TM);
-				strNcpy(q,tmpdt,freespace);
-				q += strlen(q);
+				len = strftime(tmpdt, sizeof(tmpdt), "%Y%m%d", TM);
+				if (len > 0) {
+					strNcpy(q, tmpdt, freespace);
+					q += strlen(q);
+				}
 				p++;
 				break;
 			case 'H': /* request hour */
 				TM = localtime_r(&request->timestamp, &s_TM);
-				strftime(tmpdt,sizeof(tmpdt),"%H",TM);
-				strNcpy(q,tmpdt,freespace);
-				q += strlen(q);
+				len = strftime(tmpdt, sizeof(tmpdt), "%H", TM);
+				if (len > 0) {
+					strNcpy(q, tmpdt, freespace);
+					q += strlen(q);
+				}
 				p++;
 				break;
 			case 'L': /* radlog_dir */
@@ -756,16 +763,20 @@ int radius_xlat(char *out, int outlen, const char *fmt,
 				break;
 			case 'S': /* request timestamp in SQL format*/
 				TM = localtime_r(&request->timestamp, &s_TM);
-				strftime(tmpdt,sizeof(tmpdt),"%Y-%m-%d %H:%M:%S",TM);
-				strNcpy(q,tmpdt,freespace);
-				q += strlen(q);
+				len = strftime(tmpdt, sizeof(tmpdt), "%Y-%m-%d %H:%M:%S", TM);
+				if (len > 0) {
+					strNcpy(q, tmpdt, freespace);
+					q += strlen(q);
+				}
 				p++;
 				break;
 			case 'T': /* request timestamp */
 				TM = localtime_r(&request->timestamp, &s_TM);
-				strftime(tmpdt,sizeof(tmpdt),"%Y-%m-%d-%H.%M.%S.000000",TM);
-				strNcpy(q,tmpdt,freespace);
-				q += strlen(q);
+				len = strftime(tmpdt, sizeof(tmpdt), "%Y-%m-%d-%H.%M.%S.000000", TM);
+				if (len > 0) {
+					strNcpy(q, tmpdt, freespace);
+					q += strlen(q);
+				}
 				p++;
 				break;
 			case 'U': /* Stripped User name */
@@ -782,9 +793,11 @@ int radius_xlat(char *out, int outlen, const char *fmt,
 				break;
 			case 'Y': /* request year */
 				TM = localtime_r(&request->timestamp, &s_TM);
-				strftime(tmpdt,sizeof(tmpdt),"%Y",TM);
-				strNcpy(q,tmpdt,freespace);
-				q += strlen(q);
+				len = strftime(tmpdt, sizeof(tmpdt), "%Y", TM);
+				if (len > 0) {
+					strNcpy(q, tmpdt, freespace);
+					q += strlen(q);
+				}
 				p++;
 				break;
 			case 'Z': /* Full request pairs except password */
@@ -792,9 +805,9 @@ int radius_xlat(char *out, int outlen, const char *fmt,
 				while (tmp && (freespace > 3)) {
 					if (tmp->attribute != PW_PASSWORD) {
 						*q++ = '\t';
-						i = vp_prints(q,freespace-2,tmp);
-						q += i;
-						freespace -= (i+2);
+						len = vp_prints(q, freespace - 2, tmp);
+						q += len;
+						freespace -= (len + 2);
 						*q++ = '\n';
 					}
 					tmp = tmp->next;
