@@ -1,5 +1,5 @@
 /*
- * $Id: db_postgresql.sql,v 1.16 2004/02/29 13:06:58 kkalev Exp $
+ * $Id: db_postgresql.sql,v 1.16.2.4 2005/08/23 15:40:15 nbk Exp $
  *
  * Postgresql schema for FreeRADIUS
  *
@@ -17,10 +17,10 @@ CREATE TABLE radacct (
 	RadAcctId		BIGSERIAL PRIMARY KEY,
 	AcctSessionId		VARCHAR(32) NOT NULL,
 	AcctUniqueId		VARCHAR(32) NOT NULL,
-	UserName		VARCHAR(32),
+	UserName		VARCHAR(253),
 	Realm			VARCHAR(30),
 	NASIPAddress		INET NOT NULL,
-	NASPortId		INTEGER,
+	NASPortId		BIGINT,
 	NASPortType		VARCHAR(32),
 	AcctStartTime		TIMESTAMP with time zone,
 	AcctStopTime		TIMESTAMP with time zone,
@@ -81,6 +81,10 @@ CREATE TABLE radcheck (
 	Value		VARCHAR(40)
 );
 create index radcheck_UserName on radcheck (UserName,Attribute);
+/*
+ * Use this index if you use case insensitive queries
+ */
+-- create index radcheck_UserName_lower on radcheck (lower(UserName),Attribute);
 
 /*
  * Table structure for table 'radgroupcheck'
@@ -117,6 +121,10 @@ CREATE TABLE radreply (
 	Value		VARCHAR(40)
 );
 create index radreply_UserName on radreply (UserName,Attribute);
+/*
+ * Use this index if you use case insensitive queries
+ */
+-- create index radreply_UserName_lower on radreply (lower(UserName),Attribute);
 
 /*
  * Table structure for table 'usergroup'
@@ -127,6 +135,10 @@ CREATE TABLE usergroup (
 	GroupName	VARCHAR(30)
 );
 create index usergroup_UserName on usergroup (UserName);
+/*
+ * Use this index if you use case insensitive queries
+ */
+-- create index usergroup_UserName_lower on usergroup (lower(UserName));
 
 /*
  * Table structure for table 'realmgroup'
@@ -173,8 +185,8 @@ CREATE TABLE nas (
 
 CREATE TABLE radpostauth (
 	id		BIGSERIAL PRIMARY KEY,
-	username	VARCHAR(64) NOT NULL,
-	pass		VARCHAR(64),
+	username	VARCHAR(253) NOT NULL,
+	pass		VARCHAR(128),
 	reply		VARCHAR(32),
 	authdate	TIMESTAMP with time zone NOT NULL default 'now'
 ) ;
