@@ -2,7 +2,7 @@
  * rlm_sql.c		SQL Module
  * 		Main SQL module file. Most ICRADIUS code is located in sql.c
  *
- * Version:	$Id: rlm_sql.c,v 1.131.2.3 2005/05/18 13:22:18 nbk Exp $
+ * Version:	$Id: rlm_sql.c,v 1.131.2.4 2005/08/31 12:49:52 nbk Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 
 static const char rcsid[] =
-	"$Id: rlm_sql.c,v 1.131.2.3 2005/05/18 13:22:18 nbk Exp $";
+	"$Id: rlm_sql.c,v 1.131.2.4 2005/08/31 12:49:52 nbk Exp $";
 
 #include "autoconf.h"
 
@@ -165,14 +165,14 @@ static int sql_escape_func(char *out, int outlen, const char *in);
  *	the first element of the SELECT result will be used.
  */
 static int sql_xlat(void *instance, REQUEST *request,
-		    char *fmt, char *out, int freespace,
+		    char *fmt, char *out, size_t freespace,
 		    RADIUS_ESCAPE_STRING func)
 {
 	SQLSOCK *sqlsocket;
 	SQL_ROW row;
 	SQL_INST *inst = instance;
 	char querystr[MAX_QUERY_LEN];
-	char sqlusername[2 * MAX_STRING_LEN + 10];
+	char sqlusername[MAX_STRING_LEN];
 	int ret = 0;
 
 	DEBUG("rlm_sql (%s): - sql_xlat", inst->config->xlat_name);
@@ -451,15 +451,20 @@ static int sql_escape_func(char *out, int outlen, const char *in)
 }
 
 /*
- *	Set the SQl user name.
+ *	Set the SQL user name.
+ *
+ *	We don't call the escape function here. The resulting string
+ *	will be escaped later in the queries xlat so we don't need to
+ *	escape it twice. (it will make things wrong if we have an
+ *	escape candidate character in the username)
  */
 static int sql_set_user(SQL_INST *inst, REQUEST *request, char *sqlusername, const char *username)
 {
 	VALUE_PAIR *vp=NULL;
 	char tmpuser[MAX_STRING_LEN];
 
-	tmpuser[0]=0;
-	sqlusername[0]=0;
+	tmpuser[0] = '\0';
+	sqlusername[0]= '\0';
 
 	/* Remove any user attr we added previously */
 	pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
@@ -473,7 +478,7 @@ static int sql_set_user(SQL_INST *inst, REQUEST *request, char *sqlusername, con
 	}
 
 	if (*tmpuser) {
-		strNcpy(sqlusername, tmpuser, sizeof(tmpuser));
+		strNcpy(sqlusername, tmpuser, MAX_STRING_LEN);
 		DEBUG2("rlm_sql (%s): sql_set_user escaped user --> '%s'",
 		       inst->config->xlat_name, sqlusername);
 		vp = pairmake("SQL-User-Name", sqlusername, 0);
@@ -502,7 +507,7 @@ static int sql_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE
 	SQL_ROW row;
 	SQL_INST *inst = instance;
 	char querystr[MAX_QUERY_LEN];
-	char sqlusername[2 * MAX_STRING_LEN + 10];
+	char sqlusername[MAX_STRING_LEN];
 
 	check_pairs = check_pairs;
 	reply_pairs = reply_pairs;
@@ -523,7 +528,7 @@ static int sql_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE
 	/*
 	 * Set, escape, and check the user attr here
 	 */
-	if (sql_set_user(inst, req, sqlusername, 0) < 0)
+	if (sql_set_user(inst, req, sqlusername, NULL) < 0)
 		return 1;
 	if (!radius_xlat(querystr, sizeof(querystr), inst->config->groupmemb_query, req, sql_escape_func)){
 		radlog(L_ERR, "rlm_sql (%s): xlat failed.",
@@ -739,13 +744,7 @@ static int rlm_sql_authorize(void *instance, REQUEST * request)
 	SQLSOCK *sqlsocket;
 	SQL_INST *inst = instance;
 	char    querystr[MAX_QUERY_LEN];
-
-	/* sqlusername holds the sql escaped username. The original
-	 * username is at most MAX_STRING_LEN chars long and
-	 * *sql_escape_string doubles its length in the worst case.
-	 * Throw in an extra 10 to account for trailing NULs and to have
-	 * a safety margin. */
-	char   sqlusername[2 * MAX_STRING_LEN + 10];
+	char	sqlusername[MAX_STRING_LEN];
 
 	/*
 	 *	They MUST have a user name to do SQL authorization.
@@ -897,7 +896,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 	int     acctstatustype = 0;
 	char    querystr[MAX_QUERY_LEN];
 	char    logstr[MAX_QUERY_LEN];
-	char	sqlusername[2 * MAX_STRING_LEN + 10];
+	char	sqlusername[MAX_STRING_LEN];
 
 #ifdef CISCO_ACCOUNTING_HACK
 	int     acctsessiontime = 0;
@@ -1134,7 +1133,7 @@ static int rlm_sql_checksimul(void *instance, REQUEST * request) {
 	SQL_INST	*inst = instance;
 	SQL_ROW		row;
 	char		querystr[MAX_QUERY_LEN];
-	char		sqlusername[2*MAX_STRING_LEN+10];
+	char		sqlusername[MAX_STRING_LEN];
 	int		check = 0;
         uint32_t        ipno = 0;
         char            *call_num = NULL;
@@ -1154,7 +1153,7 @@ static int rlm_sql_checksimul(void *instance, REQUEST * request) {
 	}
 
 
-	if(sql_set_user(inst, request, sqlusername, 0) <0)
+	if(sql_set_user(inst, request, sqlusername, NULL) < 0)
 		return RLM_MODULE_FAIL;
 
 	radius_xlat(querystr, sizeof(querystr), inst->config->simul_count_query, request, sql_escape_func);
@@ -1301,11 +1300,11 @@ static int rlm_sql_postauth(void *instance, REQUEST *request) {
 	SQLSOCK 	*sqlsocket = NULL;
 	SQL_INST	*inst = instance;
 	char		querystr[MAX_QUERY_LEN];
-	char		sqlusername[2*MAX_STRING_LEN+10];
+	char		sqlusername[MAX_STRING_LEN];
 
 	DEBUG("rlm_sql (%s): Processing sql_postauth", inst->config->xlat_name);
 
-	if(sql_set_user(inst, request, sqlusername, 0) <0)
+	if(sql_set_user(inst, request, sqlusername, NULL) < 0)
 		return RLM_MODULE_FAIL;
 
 	/* If postauth_query is not defined, we stop here */
