@@ -1,7 +1,7 @@
 /*
  * rlm_sqlcounter.c
  *
- * Version:  $Id: rlm_sqlcounter.c,v 1.11 2004/05/15 15:50:26 mgriego Exp $
+ * Version:  $Id: rlm_sqlcounter.c,v 1.11.2.3 2005/08/23 14:13:38 nbk Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@
  */
 
 
-static const char rcsid[] = "$Id: rlm_sqlcounter.c,v 1.11 2004/05/15 15:50:26 mgriego Exp $";
+static const char rcsid[] = "$Id: rlm_sqlcounter.c,v 1.11.2.3 2005/08/23 14:13:38 nbk Exp $";
 
 /*
  *	Define a structure for our module configuration.
@@ -97,6 +97,60 @@ static CONF_PARSER module_config[] = {
   { NULL, -1, 0, NULL, NULL }
 };
 
+/*
+ *	Safe characters list for sql queries. Everything else is
+ *	replaced with their mime-encoded equivalents.
+ */
+static const char allowed_chars[] = "@abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.-_: /";
+
+/*
+ *	Translate the SQL queries.
+ */
+static int sql_escape_func(char *out, int outlen, const char *in)
+{
+	int len = 0;
+
+	while (in[0]) {
+		/*
+		 *	Non-printable characters get replaced with their
+		 *	mime-encoded equivalents.
+		 */
+		if ((in[0] < 32) ||
+		    strchr(allowed_chars, *in) == NULL) {
+			/*
+			 *	Only 3 or less bytes available.
+			 */
+			if (outlen <= 3) {
+				break;
+			}
+
+			snprintf(out, outlen, "=%02X", (unsigned char) in[0]);
+			in++;
+			out += 3;
+			outlen -= 3;
+			len += 3;
+			continue;
+		}
+
+		/*
+		 *	Only one byte left.
+		 */
+		if (outlen <= 1) {
+			break;
+		}
+
+		/*
+		 *	Allowed character.
+		 */
+		*out = *in;
+		out++;
+		in++;
+		outlen--;
+		len++;
+	}
+	*out = '\0';
+	return len;
+}
 
 static int find_next_reset(rlm_sqlcounter_t *data, time_t timeval)
 {
@@ -294,12 +348,12 @@ static int sqlcounter_expand(char *out, int outlen, const char *fmt, void *insta
 			case '%':
 				*q++ = *p;
 			case 'b': /* last_reset */
-				sprintf(tmpdt, "%lu", data->last_reset);
+				snprintf(tmpdt, sizeof(tmpdt), "%lu", data->last_reset);
 				strNcpy(q, tmpdt, freespace);
 				q += strlen(q);
 				break;
 			case 'e': /* reset_time */
-				sprintf(tmpdt, "%lu", data->reset_time);
+				snprintf(tmpdt, sizeof(tmpdt), "%lu", data->reset_time);
 				strNcpy(q, tmpdt, freespace);
 				q += strlen(q);
 				break;
@@ -343,14 +397,14 @@ static int sqlcounter_cmp(void *instance, REQUEST *req, VALUE_PAIR *request, VAL
 	sqlcounter_expand(querystr, MAX_QUERY_LEN, data->query, instance);
 
 	/* second, xlat any request attribs in query */
-	radius_xlat(responsestr, MAX_QUERY_LEN, querystr, req, NULL);
+	radius_xlat(responsestr, MAX_QUERY_LEN, querystr, req, sql_escape_func);
 
 	/* third, wrap query with sql module call & expand */
-	sprintf(querystr, "%%{%%S:%s}", responsestr);
+	snprintf(querystr, sizeof(querystr), "%%{%%S:%s}", responsestr);
 	sqlcounter_expand(responsestr, MAX_QUERY_LEN, querystr, instance);
 
 	/* Finally, xlat resulting SQL query */
-	radius_xlat(querystr, MAX_QUERY_LEN, responsestr, req, NULL);
+	radius_xlat(querystr, MAX_QUERY_LEN, responsestr, req, sql_escape_func);
 
 	counter = atoi(querystr);
 
@@ -542,14 +596,14 @@ static int sqlcounter_authorize(void *instance, REQUEST *request)
 	sqlcounter_expand(querystr, MAX_QUERY_LEN, data->query, instance);
 
 	/* second, xlat any request attribs in query */
-	radius_xlat(responsestr, MAX_QUERY_LEN, querystr, request, NULL);
+	radius_xlat(responsestr, MAX_QUERY_LEN, querystr, request, sql_escape_func);
 
 	/* third, wrap query with sql module & expand */
-	sprintf(querystr, "%%{%%S:%s}", responsestr);
+	snprintf(querystr, sizeof(querystr), "%%{%%S:%s}", responsestr);
 	sqlcounter_expand(responsestr, MAX_QUERY_LEN, querystr, instance);
 
 	/* Finally, xlat resulting SQL query */
-	radius_xlat(querystr, MAX_QUERY_LEN, responsestr, request, NULL);
+	radius_xlat(querystr, MAX_QUERY_LEN, responsestr, request, sql_escape_func);
 
 	counter = atoi(querystr);
 
@@ -579,6 +633,7 @@ static int sqlcounter_authorize(void *instance, REQUEST *request)
 		 */
 		if (data->reset_time && (
 			res >= (data->reset_time - request->timestamp))) {
+			res = data->reset_time - request->timestamp;
 			res += check_vp->lvalue;
 		}
 
@@ -609,8 +664,8 @@ static int sqlcounter_authorize(void *instance, REQUEST *request)
 
 		/*
 		 * User is denied access, send back a reply message
-		*/
-		sprintf(msg, "Your maximum %s usage time has been reached", data->reset);
+		 */
+		snprintf(msg, sizeof(msg), "Your maximum %s usage time has been reached", data->reset);
 		reply_item=pairmake("Reply-Message", msg, T_OP_EQ);
 		pairadd(&request->reply->vps, reply_item);
 
